package com.yaohun.enderdragonwars.effect.types;

import com.yaohun.enderdragonwars.Main;
import com.yaohun.enderdragonwars.effect.GiftEffect;
import com.yaohun.enderdragonwars.game.Game;
import com.yaohun.enderdragonwars.util.RandomUtil;
import org.bukkit.Location;
import org.bukkit.Material;
import org.bukkit.Particle;
import org.bukkit.block.Block;
import org.bukkit.entity.Entity;
import org.bukkit.entity.FallingBlock;
import org.bukkit.entity.LivingEntity;
import org.bukkit.entity.Player;
import org.bukkit.potion.PotionEffect;
import org.bukkit.potion.PotionEffectType;
import org.bukkit.scheduler.BukkitRunnable;
import org.bukkit.util.Vector;

import java.util.*;

public class BlackHoleEffect extends GiftEffect {

    private static int seconds = 15;
    private boolean stop = false;

    public BlackHoleEffect(String audience) {
        super(audience);
    }

    @Override
    public void apply(Game game) {
        List<FallingBlock> fallingBlocks = new ArrayList<>();
        Location location = game.getPlayer().getLocation().add(0, 2, 0);
        new BukkitRunnable() {
            private int i = 0;

            @Override
            public void run() {
                if (i >= seconds * 2) {
                    stop = true;
                    fallingBlocks.forEach(fallingBlock -> fallingBlock.setGravity(true));
                    location.getWorld().createExplosion(location, 3.0F, false, false);
                    cancel();
                    return;
                }
                Set<Block> blocks = getRandomUpBlocks(location, 6, 10);
                for (Block block : blocks) {
                    FallingBlock fallingBlock = location.getWorld().spawnFallingBlock(block.getLocation().add(0.5, 0, 0.5), block.getBlockData());
                    fallingBlock.setDropItem(false);
                    fallingBlock.setHurtEntities(false);
                    fallingBlock.setGravity(false);
                    block.setType(Material.AIR);
                    fallingBlocks.add(fallingBlock);
                }
                i++;
            }
        }.runTaskTimer(Main.plugin, 0L, 10L);
        new BukkitRunnable() {
            private Set<UUID> uuids = new HashSet<>();

            @Override
            public void run() {
                if (stop) {
                    cancel();
                    return;
                }
                double r = 16;
                for (Entity entity : location.getWorld().getNearbyEntities(location, r, r, r)) {
                    if (entity instanceof FallingBlock) {
                        continue;
                    }
                    Location location1 = entity.getLocation();
                    Vector vector = location.toVector().subtract(location1.toVector());
                    vector.normalize();
                    vector.multiply(0.25);
                    entity.setVelocity(vector);
                    if (entity instanceof LivingEntity livingEntity) {
                        livingEntity.damage(1);
                    }
                    if (entity instanceof Player player) {
                        player.addPotionEffect(new PotionEffect(PotionEffectType.BLINDNESS, 20 * 3, 0, true, true, true));
                        player.addPotionEffect(new PotionEffect(PotionEffectType.SLOWNESS, 20 * 3, 0, true, true, true));
                    }
                }
                Iterator<FallingBlock> iterator = fallingBlocks.iterator();
                while (iterator.hasNext()) {
                    FallingBlock fallingBlock = iterator.next();
                    org.bukkit.util.Vector vector = fallingBlock.getLocation().toVector();
                    org.bukkit.util.Vector vector1 = location.toVector();
                    Vector vector2 = vector1.subtract(vector);
                    vector2.normalize();
                    vector2.multiply(0.65);
                    fallingBlock.setVelocity(vector2);
                    if (!uuids.contains(fallingBlock.getUniqueId())) {
                        if (RandomUtil.random(0.5)) {
                            uuids.add(fallingBlock.getUniqueId());
                        } else {
                            fallingBlock.remove();
                            iterator.remove();
                        }
                    }
                }
            }
        }.runTaskTimer(Main.plugin, 0L, 5L);
        new BukkitRunnable() {
            private Set<UUID> uuids = new HashSet<>();

            @Override
            public void run() {
                if (stop) {
                    cancel();
                    return;
                }
                location.getWorld().spawnParticle(Particle.CLOUD, location, 10, 1.5, 1.5, 1.5, 0.01);
                location.getWorld().spawnParticle(Particle.INFESTED, location, 10, 1.5, 1.5, 1.5, 0.01);
                location.getWorld().spawnParticle(Particle.LARGE_SMOKE, location, 10, 1.5, 1.5, 1.5, 0.01);
                location.getWorld().spawnParticle(Particle.SQUID_INK, location, 10, 1.5, 1.5, 1.5, 0.01);
                Iterator<FallingBlock> iterator = fallingBlocks.iterator();
                while (iterator.hasNext()) {
                    FallingBlock fallingBlock = iterator.next();
                    org.bukkit.util.Vector vector = fallingBlock.getLocation().toVector();
                    org.bukkit.util.Vector vector1 = location.toVector();
                    Vector vector2 = vector1.subtract(vector);
                    vector2.normalize();
                    vector2.multiply(0.65);
                    fallingBlock.setVelocity(vector2);
                    if (!uuids.contains(fallingBlock.getUniqueId())) {
                        if (RandomUtil.random(0.5)) {
                            uuids.add(fallingBlock.getUniqueId());
                        } else {
                            fallingBlock.remove();
                            iterator.remove();
                        }
                    }
                }
            }
        }.runTaskTimer(Main.plugin, 0L, 1L);
    }

    private Set<Block> getRandomUpBlocks(Location location, int size, int amount) {
        List<Block> blocks = new ArrayList<>();
        int minX = location.getBlockX() - size;
        int minY = location.getBlockY() - size * 2;
        int minZ = location.getBlockZ() - size;
        int maxX = location.getBlockX() + size;
        int maxY = location.getBlockY() + size;
        int maxZ = location.getBlockZ() + size;
        for (int x = minX; x <= maxX; x++) {
            for (int z = minZ; z <= maxZ; z++) {
                for (int y = maxY; y >= minY; y--) {
                    Block block = location.getWorld().getBlockAt(x, y, z);
                    if (!block.getType().equals(Material.AIR)) {
                        blocks.add(block);
                        break;
                    }
                }
            }
        }
        Set<Block> blocks1 = new HashSet<>();
        if (blocks.isEmpty()) {
            return blocks1;
        }
        for (int i = 0; i < amount; i++) {
            int k = RandomUtil.getRandomInt(0, blocks.size() - 1);
            Block block = blocks.remove(k);
            blocks1.add(block);
        }
        return blocks1;
    }

}
