package com.yaohun.enderdragonwars.effect.types;

import com.yaohun.enderdragonwars.Main;
import com.yaohun.enderdragonwars.effect.GiftEffect;
import com.yaohun.enderdragonwars.game.Game;
import com.yaohun.enderdragonwars.util.RandomUtil;
import org.bukkit.Material;
import org.bukkit.Particle;
import org.bukkit.Sound;
import org.bukkit.World;
import org.bukkit.entity.Allay;
import org.bukkit.entity.EntityType;
import org.bukkit.entity.Player;
import org.bukkit.event.entity.CreatureSpawnEvent;
import org.bukkit.inventory.ItemStack;
import org.bukkit.scheduler.BukkitRunnable;
import org.bukkit.util.Vector;

import java.util.ArrayList;
import java.util.List;

public class ClearInventoryEffect extends GiftEffect {

    public ClearInventoryEffect(String audience) {
        super(audience);
    }

    @Override
    public void apply(Game game) {
        World world = game.getWorld();
        Player player = game.getPlayer();
        List<Allay> allays = new ArrayList<>();
        new BukkitRunnable() {
            private int totalCount = Math.max(15, getUseableItemCount(player).size());
            private int count = 0;

            @Override
            public void run() {
                if (count > totalCount) {
                    for (Allay allay : allays) {
                        allay.getWorld().spawnParticle(Particle.WAX_OFF, allay.getLocation(), 30, 0.5, 0.5, 0.5, 0.1);
                        allay.remove();
                    }
                    player.getInventory().clear();
                    cancel();
                    return;
                }
                Allay allay = (Allay) world.spawnEntity(player.getLocation().add(0, 1, 0), EntityType.ALLAY, CreatureSpawnEvent.SpawnReason.CUSTOM, (entity) -> {
                    Allay allay1 = (Allay) entity;
                    Vector vector = player.getEyeLocation().getDirection();
                    vector.normalize();
                    vector.multiply(0.5);
                    allay1.setVelocity(vector);
                    allay1.setGlowing(true);
                    allay1.setCollidable(false);
                    allay1.setInvulnerable(true);
                    ItemStack item = takeRandomItem(player);
                    if (item == null) {
                        item = new ItemStack(Material.DIAMOND);
                    }
                    allay1.getEquipment().setItemInMainHand(item);
                });
                player.playSound(player.getLocation(), Sound.ENTITY_ITEM_PICKUP, 1, 1);
                allays.add(allay);
                count++;
            }
        }.runTaskTimer(Main.plugin, 10L, 4L);
    }

    private List<Integer> getUseableItemCount(Player player) {
        List<Integer> useableSlots = new ArrayList<>();
        for (int i = 0; i < 36; i++) {
            ItemStack item = player.getInventory().getItem(i);
            if (item != null && item.getType() != Material.AIR) {
                useableSlots.add(i);
            }
        }
        return useableSlots;
    }

    private ItemStack takeRandomItem(Player player) {
        List<Integer> useableSlots = getUseableItemCount(player);
        if (useableSlots.isEmpty()) {
            return null;
        }
        int i = RandomUtil.getRandomInt(0, useableSlots.size() - 1);
        ItemStack item = player.getInventory().getItem(useableSlots.get(i));
        player.getInventory().setItem(useableSlots.get(i), null);
        return item;
    }

}
