package com.yaohun.enderdragonwars.manager;

import com.yaohun.enderdragonwars.effect.GiftEffect;
import com.yaohun.enderdragonwars.effect.GiftQueue;
import com.yaohun.enderdragonwars.effect.types.*;
import org.bukkit.scheduler.BukkitTask;

import java.util.HashMap;
import java.util.Map;
import java.util.function.Function;

public class GiftEffectManager {

    private static Map<String, Function<String, GiftEffect>> giftEffectMap = new HashMap<>();

    private static Map<String, GiftQueue> giftQueues = new HashMap<>();

    public static void addGiftEffect(String audience, String effectId) {
        GiftEffect giftEffect = giftEffectMap.get(effectId).apply(audience);
        GiftQueue giftQueue = giftQueues.getOrDefault(effectId, new GiftQueue(giftEffect.getQueueTime()));
        giftQueue.addEffect(giftEffect);
        giftQueues.put(effectId, giftQueue);
    }

    public static void stopQueue() {
        giftQueues.values().forEach(giftQueue -> {
            BukkitTask task = giftQueue.getTask();
            if (task != null && !task.isCancelled()) {
                task.cancel();
            }
        });
    }

    public static GiftQueue getGiftQueue(String effectId) {
        return giftQueues.get(effectId);
    }

    public static Map<String, GiftQueue> getGiftQueues() {
        return giftQueues;
    }

    public static void registerAll() {
        registerGiftEffect("清理背包", ClearInventoryEffect::new);
        registerGiftEffect("火山喷发", BomBomEffect::new);
        registerGiftEffect("幸运方块", LuckyBlockEffect::new);
        registerGiftEffect("黑洞", BlackHoleEffect::new);
        registerGiftEffect("五雷轰顶", FiveLightningEffect::new);
        registerGiftEffect("装备清空", ClearInventoryEffect::new);
        registerGiftEffect("岩浆泡澡", LavaPoolEffect::new);
        registerGiftEffect("钢铁保安", SummonIronGolem::new);
        registerGiftEffect("6倍血量", AddMaxHealthEffect::new);
        registerGiftEffect("原地复活", GiveTotemEffect::new);
        registerGiftEffect("魔豆天梯", SpawnEnderPortalEffect::new);
        registerGiftEffect("世界毁灭", WorldDestroyEffect::new);
        registerGiftEffect("变大", PlayerBigEffect::new);
        registerGiftEffect("变小", PlayerSmallEffect::new);
        registerGiftEffect("安全庇护所", SpawnBuildEffect::new);
        registerGiftEffect("螺旋升天", ToHeavenEffect::new);
        registerGiftEffect("怪物军团", SummonMobs::new);
        registerGiftEffect("芜湖起飞", WuHuJump::new);
        registerGiftEffect("随机盲盒", RandomBox::new);
    }

    private static void registerGiftEffect(String id, Function<String, GiftEffect> giftEffectGetter) {
        giftEffectMap.put(id, giftEffectGetter);
    }

}
