package com.yaohun.enderdragonwars.util;

import com.yaohun.enderdragonwars.Main;
import com.yaohun.enderdragonwars.data.Point;
import com.yaohun.enderdragonwars.data.Region;
import org.bukkit.Material;
import org.bukkit.Sound;
import org.bukkit.World;
import org.bukkit.block.Block;
import org.bukkit.block.data.type.EndPortalFrame;
import org.bukkit.scheduler.BukkitRunnable;
import org.bukkit.util.Vector;

import java.util.List;

public class BuildSpawner {

    protected ISpawnModeHander spawnModeHander;
    private World world;
    private Direction initialDirection;
    private int originX;
    private int originY;
    private int originZ;
    private int chunkWight;
    private int chunkLength;
    private int ladderLength;
    private int ladderAmount;
    private int platformWidth;
    private int platformLength;
    private int levelAmount;
    private SpawnMode spawnMode;
    private int nowX;
    private int nowY;
    private int nowZ;
    private Direction nowDirection;
    private List<Material> materials;

    public BuildSpawner(World world, Direction initialDirection, int originX, int originY, int originZ, int chunkWight, int chunkLength, int ladderLength, int ladderAmount, int platformWidth, int platformLength, int levelAmount, SpawnMode spawnMode, List<Material> materials) {
        this.world = world;
        this.initialDirection = initialDirection;
        this.originX = originX;
        this.originY = originY;
        this.originZ = originZ;
        this.chunkWight = chunkWight;
        this.chunkLength = chunkLength;
        this.ladderLength = ladderLength;
        this.ladderAmount = ladderAmount;
        this.platformWidth = platformWidth;
        this.platformLength = platformLength;
        this.levelAmount = levelAmount;
        this.spawnMode = spawnMode;
        this.materials = materials;

        nowX = originX;
        nowY = originY;
        nowZ = originZ;
        nowDirection = initialDirection;
        if (spawnMode == SpawnMode.MODE_1) {
            spawnModeHander = new SpawnModeHandler_1();
        } else if (spawnMode == SpawnMode.MODE_2) {
            spawnModeHander = new SpawnModeHandler_2();
        }
    }

    public void spawn() {
        nowX = originX;
        nowY = originY;
        nowZ = originZ;
        nowDirection = initialDirection;
        if (spawnModeHander != null) {
            spawnModeHander.spawn();
        }
    }

    private void spawnPlatform(int x, int y, int z, Direction direction) {
        if (direction == Direction.EAST || direction == Direction.WEST) {
            for (int i = 0; i < platformLength; i++) {
                for (int j = 0; j < platformWidth; j++) {
                    int x1 = x + (i * chunkWight) * direction.getPlatformVector().getBlockX();
                    int z1 = z + (j * chunkLength) * direction.getPlatformVector().getBlockZ();
                    spawnPlatformChunk(x1, y, z1, direction);
                }
            }
        } else if (direction == Direction.NORTH || direction == Direction.SOUTH) {
            for (int i = 0; i < platformLength; i++) {
                for (int j = 0; j < platformWidth; j++) {
                    int x1 = x + (j * chunkLength) * direction.getPlatformVector().getBlockX();
                    int z1 = z + (i * chunkWight) * direction.getPlatformVector().getBlockZ();
                    spawnPlatformChunk(x1, y, z1, direction);
                }
            }
        }
    }

    private void spawnLadder(int x, int y, int z, Direction direction) {
        for (int i = 0; i < ladderAmount; i++) {
            for (int j = 0; j < ladderLength; j++) {
                int x1 = getLadderX(i, j, x, z, direction);
                int z1 = getLadderZ(i, j, x, z, direction);
                spawnLadderChunk(x1, y + i + 1, z1, direction);
            }
        }
    }

    private int getLadderX(int i, int j, int x, int z, Direction direction) {
        switch (direction) {
            case EAST:
            case WEST:
                return x + (i * chunkWight) * direction.getLadderVector().getBlockX();
            case SOUTH:
            case NORTH:
                return x + (j * chunkLength) * direction.getLadderVector().getBlockX();
        }
        return x;
    }

    private int getLadderZ(int i, int j, int x, int z, Direction direction) {
        switch (direction) {
            case EAST:
            case WEST:
                return z + (j * chunkLength) * direction.getLadderVector().getBlockZ();
            case SOUTH:
            case NORTH:
                return z + (i * chunkWight) * direction.getLadderVector().getBlockZ();
        }
        return z;
    }

    private void spawnPlatformChunk(int x, int y, int z, Direction direction) {
        Material material = getRandomMaterial();
        if (direction == Direction.EAST || direction == Direction.WEST) {
            for (int i = 0; i < chunkWight; i++) {
                for (int j = 0; j < chunkLength; j++) {
                    int x1 = x + i * direction.getPlatformVector().getBlockX();
                    int z1 = z + j * direction.getPlatformVector().getBlockZ();
                    world.getBlockAt(x1, y, z1).setType(material);
                }
            }
        } else if (direction == Direction.NORTH || direction == Direction.SOUTH) {
            for (int i = 0; i < chunkWight; i++) {
                for (int j = 0; j < chunkLength; j++) {
                    int x1 = x + j * direction.getPlatformVector().getBlockX();
                    int z1 = z + i * direction.getPlatformVector().getBlockZ();
                    world.getBlockAt(x1, y, z1).setType(material);
                }
            }
        }
    }

    private void spawnLadderChunk(int x, int y, int z, Direction direction) {
        Material material = getRandomMaterial();
        if (direction == Direction.EAST || direction == Direction.WEST) {
            for (int i = 0; i < chunkWight; i++) {
                for (int j = 0; j < chunkLength; j++) {
                    int x1 = x + i * direction.getLadderVector().getBlockX();
                    int z1 = z + j * direction.getLadderVector().getBlockZ();
                    world.getBlockAt(x1, y, z1).setType(material);
                }
            }
        } else if (direction == Direction.NORTH || direction == Direction.SOUTH) {
            for (int i = 0; i < chunkLength; i++) {
                for (int j = 0; j < chunkWight; j++) {
                    int x1 = x + i * direction.getLadderVector().getBlockX();
                    int z1 = z + j * direction.getLadderVector().getBlockZ();
                    world.getBlockAt(x1, y, z1).setType(material);
                }
            }
        }
    }

    private void transformDirection() {
        nowDirection = nowDirection.getRightDirection();
    }

    public Region getEffectiveRegion(World world) {
        return spawnModeHander.getEffectiveRegion(world);
    }

    public int getHeight() {
        return spawnModeHander.getHeight();
    }

    public void setInitialDirection(Direction initialDirection) {
        this.initialDirection = initialDirection;
    }

    public Material getRandomMaterial() {
        return materials.get(RandomUtil.getRandomInt(0, materials.size() - 1));
    }

    private enum TransformType {
        PLATFORM,
        LADDER
    }

    public enum Direction {

        NORTH(new Vector(1.0, 1.0, -1.0), new Vector(-1.0, 1.0, -1.0), new Vector(1.0, 1.0, -1.0)),
        EAST(new Vector(1.0, 1.0, 1.0), new Vector(1.0, 1.0, -1.0), new Vector(1.0, 1.0, 1.0)),
        SOUTH(new Vector(-1.0, 1.0, 1.0), new Vector(1.0, 1.0, 1.0), new Vector(-1.0, 1.0, 1.0)),
        WEST(new Vector(-1.0, 1.0, -1.0), new Vector(-1.0, 1.0, 1.0), new Vector(-1.0, 1.0, -1.0));

        private final Vector vector;
        private final Vector platformVector;
        private final Vector ladderVector;

        Direction(Vector vector, Vector platformVector, Vector ladderVector) {
            this.vector = vector;
            this.platformVector = platformVector;
            this.ladderVector = ladderVector;
        }

        public Vector getVector() {
            return vector;
        }

        public Vector getPlatformVector() {
            return platformVector;
        }

        public Vector getLadderVector() {
            return ladderVector;
        }

        public Direction getRightDirection() {
            switch (this) {
                case NORTH:
                    return EAST;
                case EAST:
                    return SOUTH;
                case SOUTH:
                    return WEST;
                case WEST:
                    return NORTH;
            }
            return null;
        }

    }

    public enum SpawnMode {

        MODE_1,
        MODE_2

    }

    private interface ISpawnModeHander {
        void spawn();

        Region getEffectiveRegion(World world);

        int getHeight();
    }

    private class SpawnModeHandler_1 implements ISpawnModeHander {
        @Override
        public void spawn() {
            new BukkitRunnable() {
                private int count = 0;

                @Override
                public void run() {
                    if (count >= levelAmount) {
                        spawnPlatform(nowX, nowY, nowZ, nowDirection);
                        cancel();
                        return;
                    }
                    spawnPlatform(nowX, nowY, nowZ, nowDirection);
                    transformXYZ(TransformType.PLATFORM, nowDirection);
                    transformDirection();
                    spawnLadder(nowX, nowY, nowZ, nowDirection);
                    transformXYZ(TransformType.LADDER, nowDirection);
                    transformDirection();
                    count++;
                }
            }.runTaskTimer(Main.plugin, 0L, 3L);
        }

        @Override
        public Region getEffectiveRegion(World world) {
            if (initialDirection == Direction.NORTH || initialDirection == Direction.SOUTH) {
                int minX = originX + (platformWidth * chunkWight) * initialDirection.getPlatformVector().getBlockX();
                int minY = originY;
                int minZ = originZ;
                int maxX = originX + ladderAmount * ladderLength * chunkLength * initialDirection.getLadderVector().getBlockX() + (platformWidth * chunkWight) * initialDirection.getPlatformVector().getBlockX() + 1;
                int maxY = originY + levelAmount * 5 + 1;
                int maxZ = originZ + (platformLength * chunkLength) * initialDirection.getPlatformVector().getBlockZ() + 1;
                return new Region(world, new Point(minX, minY, minZ), new Point(maxX, maxY, maxZ));
            } else if (initialDirection == Direction.EAST || initialDirection == Direction.WEST) {
                int minX = originX;
                int minY = originY;
                int minZ = originZ + (platformWidth * chunkWight) * initialDirection.getPlatformVector().getBlockZ();
                int maxX = originX + (platformLength * chunkLength) * initialDirection.getPlatformVector().getBlockX() + 1;
                int maxY = originY + levelAmount * 5 + 1;
                int maxZ = originZ + ladderAmount * ladderLength * chunkLength * initialDirection.getLadderVector().getBlockX() + (platformWidth * chunkWight) * initialDirection.getPlatformVector().getBlockZ() + 1;
                return new Region(world, new Point(minX, minY, minZ), new Point(maxX, maxY, maxZ));
            }
            return null;
        }

        @Override
        public int getHeight() {
            return levelAmount * 5;
        }

        private void transformXYZ(TransformType transformType, Direction direction) {
            if (transformType == TransformType.PLATFORM) {
                if (direction == Direction.EAST || direction == Direction.WEST) {
                    nowX += (platformLength * chunkWight - 1) * direction.getPlatformVector().getX();
                } else if (direction == Direction.NORTH || direction == Direction.SOUTH) {
                    nowZ += (platformLength * chunkWight - 1) * direction.getPlatformVector().getZ();
                }
                switch (nowDirection) {
                    case EAST:
                        nowZ += 1;
                        break;
                    case SOUTH:
                        nowX -= 1;
                        break;
                    case WEST:
                        nowZ -= 1;
                        break;
                    case NORTH:
                        nowX += 1;
                        break;
                }
            } else if (transformType == TransformType.LADDER) {
                nowY += ladderAmount;
                if (direction == Direction.EAST || direction == Direction.WEST) {
                    nowX += (ladderAmount * chunkWight) * direction.getLadderVector().getBlockX();
                } else if (direction == Direction.NORTH || direction == Direction.SOUTH) {
                    nowZ += (ladderAmount * chunkWight) * direction.getLadderVector().getBlockZ();
                }
            }
        }
    }

    private class SpawnModeHandler_2 implements ISpawnModeHander {

        @Override
        public void spawn() {
            new BukkitRunnable() {
                private int count = 0;

                @Override
                public void run() {
                    if (count >= levelAmount) {
                        spawnPlatform(nowX, nowY, nowZ, nowDirection);
                        Region region = getEffectiveRegion(world);
                        int centerX = (region.getMax().getBlockX() - region.getMin().getBlockX() + 1) / 2;
                        int centerZ = (region.getMax().getBlockZ() - region.getMin().getBlockZ() + 1) / 2;
                        Point blockCenterPoint = new Point(region.getMin().getBlockX() + centerZ, region.getMax().getBlockY() - 3, region.getMin().getBlockZ() + centerZ);
                        Block block = world.getBlockAt(blockCenterPoint.getBlockX(), blockCenterPoint.getBlockY(), blockCenterPoint.getBlockZ());
                        for (int i = -6; i <= 6; i++) {
                            for (int j = -6; j <= 6; j++) {
                                if (Math.abs(i) == 6 && Math.abs(j) == 6) {
                                    continue;
                                }
                                Block block1 = world.getBlockAt(blockCenterPoint.getBlockX() + i, blockCenterPoint.getBlockY(), blockCenterPoint.getBlockZ() + j);
                                if (Math.abs(i) <= 5 && Math.abs(j) <= 5) {
                                    block1.setType(Material.END_PORTAL);
                                } else {
                                    block1.setType(Material.END_PORTAL_FRAME);
                                    EndPortalFrame endPortalFrame = (EndPortalFrame) block1.getBlockData();
                                    endPortalFrame.setEye(true);
                                    block1.setBlockData(endPortalFrame);
                                }
                            }
                        }
                        block.getWorld().strikeLightning(block.getLocation().add(0.5, 0, 0.5));
                        block.getWorld().getPlayers().forEach(player -> player.playSound(player.getLocation(), Sound.BLOCK_END_PORTAL_SPAWN, 1.0f, 1.0f));
                        cancel();
                        return;
                    }
                    spawnPlatform(nowX, nowY, nowZ, nowDirection);
                    transformXYZ(TransformType.PLATFORM, nowDirection, 0);
                    spawnLadder(nowX, nowY, nowZ, nowDirection);
                    transformXYZ(TransformType.LADDER, nowDirection, 0);
                    transformDirection();
                    count++;
                }
            }.runTaskTimer(Main.plugin, 0L, 3L);
        }

        @Override
        public Region getEffectiveRegion(World world) {
            if (initialDirection == Direction.NORTH || initialDirection == Direction.SOUTH) {
                int minX = originX + (platformWidth * chunkWight) * initialDirection.getPlatformVector().getBlockX();
                int minZ = originZ;
                minZ += (platformLength * chunkLength) * initialDirection.getPlatformVector().getBlockZ();
                minZ += (ladderAmount * chunkLength) * initialDirection.getLadderVector().getBlockZ();
                minZ += (platformWidth * chunkWight) * initialDirection.getRightDirection().getPlatformVector().getBlockZ();
                int maxX = originX + ladderAmount * ladderLength * chunkLength * initialDirection.getLadderVector().getBlockX() + (platformWidth * chunkWight) * initialDirection.getPlatformVector().getBlockX() + 1;
                int maxZ = originZ;
                int maxY = originY + levelAmount * 5 + 1;
                return new Region(world, new Point(minX, originY, minZ), new Point(maxX, maxY, maxZ));
            } else if (initialDirection == Direction.EAST || initialDirection == Direction.WEST) {
                int minX = originX;
                int minZ = originZ + (platformWidth * chunkWight) * initialDirection.getPlatformVector().getBlockZ();
                minX += (platformLength * chunkLength) * initialDirection.getPlatformVector().getBlockX();
                minX += (ladderAmount * chunkLength) * initialDirection.getLadderVector().getBlockX();
                minX += (platformWidth * chunkWight) * initialDirection.getRightDirection().getPlatformVector().getBlockX();
                int maxX = originX;
                int maxZ = originZ + ladderAmount * ladderLength * chunkLength * initialDirection.getLadderVector().getBlockZ() + (platformWidth * chunkWight) * initialDirection.getPlatformVector().getBlockZ() + 1;
                int maxY = originY + levelAmount * 5 + 1;
                return new Region(world, new Point(minX, originY, minZ), new Point(maxX, maxY, maxZ));
            }
            return null;
        }

        @Override
        public int getHeight() {
            return levelAmount * 5;
        }

        private void transformXYZ(TransformType transformType, Direction direction, int type) {
            if (transformType == TransformType.PLATFORM) {
                if (type == 0) {
                    if (direction == Direction.EAST || direction == Direction.WEST) {
                        nowX += (platformLength * chunkWight) * direction.getPlatformVector().getX();
                        nowZ += (platformWidth * chunkLength - 1) * direction.getPlatformVector().getZ();
                    } else if (direction == Direction.NORTH || direction == Direction.SOUTH) {
                        nowX += (platformWidth * chunkLength - 1) * direction.getPlatformVector().getX();
                        nowZ += (platformLength * chunkWight) * direction.getPlatformVector().getZ();
                    }
                } else if (type == 1) {
                    if (direction == Direction.EAST || direction == Direction.WEST) {
                        nowX += (platformLength * chunkWight - 1) * direction.getPlatformVector().getX();
                    } else if (direction == Direction.NORTH || direction == Direction.SOUTH) {
                        nowZ += (platformLength * chunkWight - 1) * direction.getPlatformVector().getZ();
                    }
                    switch (nowDirection) {
                        case EAST:
                            nowZ += 1;
                            break;
                        case SOUTH:
                            nowX -= 1;
                            break;
                        case WEST:
                            nowZ -= 1;
                            break;
                        case NORTH:
                            nowX += 1;
                            break;
                    }
                }
            } else if (transformType == TransformType.LADDER) {
                nowY += ladderAmount;
                if (direction == Direction.EAST || direction == Direction.WEST) {
                    nowX += (ladderAmount * chunkWight) * direction.getLadderVector().getBlockX();
                } else if (direction == Direction.NORTH || direction == Direction.SOUTH) {
                    nowZ += (ladderAmount * chunkWight) * direction.getLadderVector().getBlockZ();
                }
            }
        }

    }

}
